@@ -1,8 +1,8 @@
 use core::convert::Infallible;
 
 use super::{
-    dynamic::PinModeError, DynamicPin, ErasedPin, Floating, Input, OpenDrain, Output,
-    PartiallyErasedPin, Pin, PinState, PullDown, PullUp, PushPull,
+    dynamic::PinModeError, marker, DynamicPin, ErasedPin, Input, OpenDrain, Output,
+    PartiallyErasedPin, Pin, PinMode, PinState,
 };
 
 use embedded_hal::digital::v2::{
@@ -11,7 +11,7 @@ use embedded_hal::digital::v2::{
 
 // Implementations for `Pin`
 
-impl<MODE, const P: char, const N: u8> OutputPin for Pin<Output<MODE>, P, N> {
+impl<const P: char, const N: u8, MODE> OutputPin for Pin<P, N, Output<MODE>> {
     type Error = Infallible;
 
     #[inline(always)]
@@ -27,7 +27,7 @@ impl<MODE, const P: char, const N: u8> OutputPin for Pin<Output<MODE>, P, N> {
     }
 }
 
-impl<MODE, const P: char, const N: u8> StatefulOutputPin for Pin<Output<MODE>, P, N> {
+impl<const P: char, const N: u8, MODE> StatefulOutputPin for Pin<P, N, Output<MODE>> {
     #[inline(always)]
     fn is_set_high(&self) -> Result<bool, Self::Error> {
         Ok(self.is_set_high())
@@ -39,7 +39,7 @@ impl<MODE, const P: char, const N: u8> StatefulOutputPin for Pin<Output<MODE>, P
     }
 }
 
-impl<MODE, const P: char, const N: u8> ToggleableOutputPin for Pin<Output<MODE>, P, N> {
+impl<const P: char, const N: u8, MODE> ToggleableOutputPin for Pin<P, N, Output<MODE>> {
     type Error = Infallible;
 
     #[inline(always)]
@@ -49,7 +49,10 @@ impl<MODE, const P: char, const N: u8> ToggleableOutputPin for Pin<Output<MODE>,
     }
 }
 
-impl<const P: char, const N: u8> InputPin for Pin<Output<OpenDrain>, P, N> {
+impl<const P: char, const N: u8, MODE> InputPin for Pin<P, N, MODE>
+where
+    MODE: marker::Readable,
+{
     type Error = Infallible;
 
     #[inline(always)]
@@ -63,21 +66,7 @@ impl<const P: char, const N: u8> InputPin for Pin<Output<OpenDrain>, P, N> {
     }
 }
 
-impl<MODE, const P: char, const N: u8> InputPin for Pin<Input<MODE>, P, N> {
-    type Error = Infallible;
-
-    #[inline(always)]
-    fn is_high(&self) -> Result<bool, Self::Error> {
-        Ok(self.is_high())
-    }
-
-    #[inline(always)]
-    fn is_low(&self) -> Result<bool, Self::Error> {
-        Ok(self.is_low())
-    }
-}
-
-impl<const P: char, const N: u8> IoPin<Self, Self> for Pin<Output<OpenDrain>, P, N> {
+impl<const P: char, const N: u8> IoPin<Self, Self> for Pin<P, N, Output<OpenDrain>> {
     type Error = Infallible;
     fn into_input_pin(self) -> Result<Self, Self::Error> {
         Ok(self)
@@ -88,12 +77,13 @@ impl<const P: char, const N: u8> IoPin<Self, Self> for Pin<Output<OpenDrain>, P,
     }
 }
 
-impl<const P: char, const N: u8> IoPin<Pin<Input<Floating>, P, N>, Self>
-    for Pin<Output<OpenDrain>, P, N>
+impl<const P: char, const N: u8, Otype> IoPin<Pin<P, N, Input>, Self> for Pin<P, N, Output<Otype>>
+where
+    Output<Otype>: PinMode,
 {
     type Error = Infallible;
-    fn into_input_pin(self) -> Result<Pin<Input<Floating>, P, N>, Self::Error> {
-        Ok(self.into_floating_input())
+    fn into_input_pin(self) -> Result<Pin<P, N, Input>, Self::Error> {
+        Ok(self.into_input())
     }
     fn into_output_pin(mut self, state: PinState) -> Result<Self, Self::Error> {
         self.set_state(state);
@@ -101,90 +91,17 @@ impl<const P: char, const N: u8> IoPin<Pin<Input<Floating>, P, N>, Self>
     }
 }
 
-impl<const P: char, const N: u8> IoPin<Self, Pin<Output<OpenDrain>, P, N>>
-    for Pin<Input<Floating>, P, N>
+impl<const P: char, const N: u8, Otype> IoPin<Self, Pin<P, N, Output<Otype>>> for Pin<P, N, Input>
+where
+    Output<Otype>: PinMode,
 {
     type Error = Infallible;
     fn into_input_pin(self) -> Result<Self, Self::Error> {
         Ok(self)
     }
-    fn into_output_pin(self, state: PinState) -> Result<Pin<Output<OpenDrain>, P, N>, Self::Error> {
-        Ok(self.into_open_drain_output_in_state(state))
-    }
-}
-
-impl<const P: char, const N: u8> IoPin<Pin<Input<Floating>, P, N>, Self>
-    for Pin<Output<PushPull>, P, N>
-{
-    type Error = Infallible;
-    fn into_input_pin(self) -> Result<Pin<Input<Floating>, P, N>, Self::Error> {
-        Ok(self.into_floating_input())
-    }
-    fn into_output_pin(mut self, state: PinState) -> Result<Self, Self::Error> {
-        self.set_state(state);
-        Ok(self)
-    }
-}
-
-impl<const P: char, const N: u8> IoPin<Self, Pin<Output<PushPull>, P, N>>
-    for Pin<Input<Floating>, P, N>
-{
-    type Error = Infallible;
-    fn into_input_pin(self) -> Result<Self, Self::Error> {
-        Ok(self)
-    }
-    fn into_output_pin(self, state: PinState) -> Result<Pin<Output<PushPull>, P, N>, Self::Error> {
-        Ok(self.into_push_pull_output_in_state(state))
-    }
-}
-
-impl<const P: char, const N: u8> IoPin<Pin<Input<PullUp>, P, N>, Self>
-    for Pin<Output<PushPull>, P, N>
-{
-    type Error = Infallible;
-    fn into_input_pin(self) -> Result<Pin<Input<PullUp>, P, N>, Self::Error> {
-        Ok(self.into_pull_up_input())
-    }
-    fn into_output_pin(mut self, state: PinState) -> Result<Self, Self::Error> {
-        self.set_state(state);
-        Ok(self)
-    }
-}
-
-impl<const P: char, const N: u8> IoPin<Self, Pin<Output<PushPull>, P, N>>
-    for Pin<Input<PullUp>, P, N>
-{
-    type Error = Infallible;
-    fn into_input_pin(self) -> Result<Self, Self::Error> {
-        Ok(self)
-    }
-    fn into_output_pin(self, state: PinState) -> Result<Pin<Output<PushPull>, P, N>, Self::Error> {
-        Ok(self.into_push_pull_output_in_state(state))
-    }
-}
-
-impl<const P: char, const N: u8> IoPin<Pin<Input<PullDown>, P, N>, Self>
-    for Pin<Output<PushPull>, P, N>
-{
-    type Error = Infallible;
-    fn into_input_pin(self) -> Result<Pin<Input<PullDown>, P, N>, Self::Error> {
-        Ok(self.into_pull_down_input())
-    }
-    fn into_output_pin(mut self, state: PinState) -> Result<Self, Self::Error> {
-        self.set_state(state);
-        Ok(self)
-    }
-}
-
-impl<const P: char, const N: u8> IoPin<Self, Pin<Output<PushPull>, P, N>>
-    for Pin<Input<PullDown>, P, N>
-{
-    type Error = Infallible;
-    fn into_input_pin(self) -> Result<Self, Self::Error> {
-        Ok(self)
-    }
-    fn into_output_pin(self, state: PinState) -> Result<Pin<Output<PushPull>, P, N>, Self::Error> {
-        Ok(self.into_push_pull_output_in_state(state))
+    fn into_output_pin(mut self, state: PinState) -> Result<Pin<P, N, Output<Otype>>, Self::Error> {
+        self._set_state(state);
+        Ok(self.into_mode())
     }
 }
 
@@ -228,21 +145,10 @@ impl<MODE> ToggleableOutputPin for ErasedPin<Output<MODE>> {
     }
 }
 
-impl InputPin for ErasedPin<Output<OpenDrain>> {
-    type Error = core::convert::Infallible;
-
-    #[inline(always)]
-    fn is_high(&self) -> Result<bool, Self::Error> {
-        Ok(self.is_high())
-    }
-
-    #[inline(always)]
-    fn is_low(&self) -> Result<bool, Self::Error> {
-        Ok(self.is_low())
-    }
-}
-
-impl<MODE> InputPin for ErasedPin<Input<MODE>> {
+impl<MODE> InputPin for ErasedPin<MODE>
+where
+    MODE: marker::Readable,
+{
     type Error = core::convert::Infallible;
 
     #[inline(always)]
@@ -258,7 +164,7 @@ impl<MODE> InputPin for ErasedPin<Input<MODE>> {
 
 // Implementations for `PartiallyErasedPin`
 
-impl<MODE, const P: char> OutputPin for PartiallyErasedPin<Output<MODE>, P> {
+impl<const P: char, MODE> OutputPin for PartiallyErasedPin<P, Output<MODE>> {
     type Error = Infallible;
 
     #[inline(always)]
@@ -274,7 +180,7 @@ impl<MODE, const P: char> OutputPin for PartiallyErasedPin<Output<MODE>, P> {
     }
 }
 
-impl<MODE, const P: char> StatefulOutputPin for PartiallyErasedPin<Output<MODE>, P> {
+impl<const P: char, MODE> StatefulOutputPin for PartiallyErasedPin<P, Output<MODE>> {
     #[inline(always)]
     fn is_set_high(&self) -> Result<bool, Self::Error> {
         Ok(self.is_set_high())
@@ -286,7 +192,7 @@ impl<MODE, const P: char> StatefulOutputPin for PartiallyErasedPin<Output<MODE>,
     }
 }
 
-impl<MODE, const P: char> ToggleableOutputPin for PartiallyErasedPin<Output<MODE>, P> {
+impl<const P: char, MODE> ToggleableOutputPin for PartiallyErasedPin<P, Output<MODE>> {
     type Error = Infallible;
 
     #[inline(always)]
@@ -296,21 +202,10 @@ impl<MODE, const P: char> ToggleableOutputPin for PartiallyErasedPin<Output<MODE
     }
 }
 
-impl<const P: char> InputPin for PartiallyErasedPin<Output<OpenDrain>, P> {
-    type Error = Infallible;
-
-    #[inline(always)]
-    fn is_high(&self) -> Result<bool, Self::Error> {
-        Ok(self.is_high())
-    }
-
-    #[inline(always)]
-    fn is_low(&self) -> Result<bool, Self::Error> {
-        Ok(self.is_low())
-    }
-}
-
-impl<MODE, const P: char> InputPin for PartiallyErasedPin<Input<MODE>, P> {
+impl<const P: char, MODE> InputPin for PartiallyErasedPin<P, MODE>
+where
+    MODE: marker::Readable,
+{
     type Error = Infallible;
 
     #[inline(always)]
